@@ -6,7 +6,7 @@ import os
 import shutil
 from collections.abc import Container, Iterable, Iterator, Sequence
 from contextlib import suppress
-from typing import Any, NamedTuple, Optional
+from typing import Any, NamedTuple, TypedDict
 
 from .boss import Boss
 from .child import Child
@@ -20,11 +20,6 @@ from .types import OverlayType, run_once
 from .utils import get_editor, log_error, resolve_custom_file, which
 from .window import CwdRequest, CwdRequestType, Watchers, Window
 
-try:
-    from typing import TypedDict
-except ImportError:
-    TypedDict = dict
-
 
 class LaunchSpec(NamedTuple):
     opts: LaunchCLIOptions
@@ -63,17 +58,24 @@ This option was added to kitty in version 0.26.0
 
 @run_once
 def options_spec() -> str:
-    return f'''
+    return f"""
+--source-window
+A match expression to select the window from which data such as title, colors, env vars,
+screen contents, etc. are copied. When not specified the currently active window is used.
+See :ref:`search_syntax` for the syntax used for specifying windows.
+
+
 --window-title --title
 The title to set for the new window. By default, title is controlled by the
 child process. The special value :code:`current` will copy the title from the
-currently active window.
+:option:`source window <launch --source-window>`.
 
 
 --tab-title
 The title for the new tab if launching in a new tab. By default, the title
 of the active window in the tab is used as the tab title. The special value
-:code:`current` will copy the title from the title of the currently active tab.
+:code:`current` will copy the title from the tab containing the
+:option:`source window <launch --source-window>`.
 
 
 --type
@@ -86,10 +88,12 @@ Where to launch the child process:
     A new :term:`kitty window <window>` in the current tab
 
 :code:`tab`
-    A new :term:`tab` in the current OS window
+    A new :term:`tab` in the current OS window. Not available when the
+    :doc:`launch <launch>` command is used in :ref:`startup sessions <sessions>`.
 
 :code:`os-window`
-    A new :term:`operating system window <os_window>`
+    A new :term:`operating system window <os_window>`.  Not available when the
+    :doc:`launch <launch>` command is used in :ref:`startup sessions <sessions>`.
 
 :code:`overlay`
     An :term:`overlay window <overlay>` covering the current active kitty window
@@ -124,7 +128,7 @@ opened window.
 --cwd
 completion=type:directory kwds:current,oldest,last_reported,root
 The working directory for the newly launched child. Use the special value
-:code:`current` to use the working directory of the currently active window.
+:code:`current` to use the working directory of the :option:`source window <launch --source-window>`
 The special value :code:`last_reported` uses the last working directory reported
 by the shell (needs :ref:`shell_integration` to work). The special value
 :code:`oldest` works like :code:`current` but uses the working directory of the
@@ -152,18 +156,18 @@ Keep the window open even after the command being executed exits, at a shell pro
 --copy-colors
 type=bool-set
 Set the colors of the newly created window to be the same as the colors in the
-currently active window.
+:option:`source window <launch --source-window>`.
 
 
 --copy-cmdline
 type=bool-set
 Ignore any specified command line and instead use the command line from the
-currently active window.
+:option:`source window <launch --source-window>`.
 
 
 --copy-env
 type=bool-set
-Copy the environment variables from the currently active window into the newly
+Copy the environment variables from the :option:`source window <launch --source-window>` into the newly
 launched child process. Note that this only copies the environment when the
 window was first created, as it is not possible to get updated environment variables
 from arbitrary processes. To copy that environment, use either the :ref:`clone-in-kitty
@@ -183,7 +187,17 @@ instead of at the end. The values of :code:`vsplit`, :code:`hsplit` and
 :code:`split` are only used by the :code:`splits` layout and control if the new
 window is placed in a vertical, horizontal or automatic split with the currently
 active window. The default is to place the window in a layout dependent manner,
-typically, after the currently active window.
+typically, after the currently active window. See :option:`--next-to <launch --next-to>`
+to use a window other than the currently active window.
+
+
+--next-to
+A match expression to select the window next to which the new window is created.
+See :ref:`search_syntax` for the syntax for specifying windows. If not specified
+defaults to the active window. When used via remote control and a target tab is
+specified this option is ignored unless the matched window is in the specified tab.
+When using :option:`--type <launch --type>` of :code:`tab`, the tab will be created
+in the OS Window containing the matched window.
 
 
 --bias
@@ -246,16 +260,16 @@ choices=none,@selection,@screen,@screen_scrollback,@alternate,@alternate_scrollb
 Pass the screen contents as :file:`STDIN` to the child process.
 
 :code:`@selection`
-    is the currently selected text.
+    is the currently selected text in the :option:`source window <launch --source-window>`.
 
 :code:`@screen`
-    is the contents of the currently active window.
+    is the contents of the :option:`source window <launch --source-window>`.
 
 :code:`@screen_scrollback`
     is the same as :code:`@screen`, but includes the scrollback buffer as well.
 
 :code:`@alternate`
-    is the secondary screen of the current active window. For example if you run
+    is the secondary screen of the :option:`source window <launch --source-window>`. For example if you run
     a full screen terminal application, the secondary screen will
     be the screen you return to when quitting the application.
 
@@ -307,7 +321,8 @@ using :option:`--type=os-window <launch --type>`. Defaults to
 --os-window-title
 Set the title for the newly created OS window. This title will override any
 titles set by programs running in kitty. The special value :code:`current` will
-use the title of the current OS window, if any.
+copy the title from the OS Window containing the
+:option:`source window <launch --source-window>`.
 
 
 --os-window-state
@@ -362,10 +377,10 @@ the section on watchers in the launch command documentation: :ref:`watchers`.
 Relative paths are resolved relative to the :ref:`kitty config directory
 <confloc>`. Global watchers for all windows can be specified with
 :opt:`watcher` in :file:`kitty.conf`.
-'''
+"""
 
 
-def parse_launch_args(args: Optional[Sequence[str]] = None) -> LaunchSpec:
+def parse_launch_args(args: Sequence[str] | None = None) -> LaunchSpec:
     args = list(args or ())
     try:
         opts, args = parse_args(result_class=LaunchCLIOptions, args=args, ospec=options_spec)
@@ -374,7 +389,7 @@ def parse_launch_args(args: Optional[Sequence[str]] = None) -> LaunchSpec:
     return LaunchSpec(opts, args)
 
 
-def get_env(opts: LaunchCLIOptions, active_child: Optional[Child] = None, base_env: Optional[dict[str,str]] = None) -> dict[str, str]:
+def get_env(opts: LaunchCLIOptions, active_child: Child | None = None, base_env: dict[str,str] | None = None) -> dict[str, str]:
     env: dict[str, str] = {}
     if opts.copy_env and active_child:
         env.update(active_child.foreground_environ)
@@ -386,9 +401,9 @@ def get_env(opts: LaunchCLIOptions, active_child: Optional[Child] = None, base_e
     return env
 
 
-def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Optional[Tab] = None) -> Optional[Tab]:
+def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Tab | None, next_to: Window | None) -> Tab:
 
-    def create_tab(tm: Optional[TabManager] = None) -> Tab:
+    def create_tab(tm: TabManager | None = None) -> Tab:
         if tm is None:
             oswid = boss.add_os_window(
                 wclass=opts.os_window_class,
@@ -402,27 +417,34 @@ def tab_for_window(boss: Boss, opts: LaunchCLIOptions, target_tab: Optional[Tab]
         return tab
 
     if opts.type == 'tab':
+        tm = boss.active_tab_manager
+        if next_to is not None and (tab := next_to.tabref()) is not None and (qtm := tab.tab_manager_ref()) is not None:
+            tm = qtm
         if target_tab is not None:
-            tm = target_tab.tab_manager_ref() or boss.active_tab_manager
-        else:
-            tm = boss.active_tab_manager
+            tm = target_tab.tab_manager_ref() or tm
         tab = create_tab(tm)
     elif opts.type == 'os-window':
         tab = create_tab()
     else:
-        tab = target_tab or boss.active_tab or create_tab()
-
+        if target_tab is not None:
+            tab = target_tab
+        else:
+            tab = boss.active_tab
+            if next_to is not None and (qtab := next_to.tabref()) is not None:
+                tab = qtab
+            tab = tab or create_tab()
     return tab
 
 
 watcher_modules: dict[str, Any] = {}
 
 
-def load_watch_modules(watchers: Iterable[str]) -> Optional[Watchers]:
+def load_watch_modules(watchers: Iterable[str]) -> Watchers | None:
     if not watchers:
         return None
     import runpy
     ans = Watchers()
+    boss = get_boss()
     for path in watchers:
         path = resolve_custom_file(path)
         m = watcher_modules.get(path, None)
@@ -436,6 +458,14 @@ def load_watch_modules(watchers: Iterable[str]) -> Optional[Watchers]:
                 watcher_modules[path] = False
                 continue
             watcher_modules[path] = m
+            w = m.get('on_load')
+            if callable(w):
+                try:
+                    w(boss, {})
+                except Exception as err:
+                    import traceback
+                    log_error(traceback.format_exc())
+                    log_error(f'Failed to call on_load() in watcher from {path} with error: {err}')
         if m is False:
             continue
         w = m.get('on_close')
@@ -456,31 +486,34 @@ def load_watch_modules(watchers: Iterable[str]) -> Optional[Watchers]:
         w = m.get('on_cmd_startstop')
         if callable(w):
             ans.on_cmd_startstop.append(w)
+        w = m.get('on_color_scheme_preference_change')
+        if callable(w):
+            ans.on_color_scheme_preference_change.append(w)
     return ans
 
 
 class LaunchKwds(TypedDict):
 
     allow_remote_control: bool
-    remote_control_passwords: Optional[dict[str, Sequence[str]]]
-    cwd_from: Optional[CwdRequest]
-    cwd: Optional[str]
-    location: Optional[str]
-    override_title: Optional[str]
-    copy_colors_from: Optional[Window]
-    marker: Optional[str]
-    cmd: Optional[list[str]]
-    overlay_for: Optional[int]
-    stdin: Optional[bytes]
+    remote_control_passwords: dict[str, Sequence[str]] | None
+    cwd_from: CwdRequest | None
+    cwd: str | None
+    location: str | None
+    override_title: str | None
+    copy_colors_from: Window | None
+    marker: str | None
+    cmd: list[str] | None
+    overlay_for: int | None
+    stdin: bytes | None
     hold: bool
-    bias: Optional[float]
+    bias: float | None
 
 
 def apply_colors(window: Window, spec: Sequence[str]) -> None:
-    from kitty.rc.set_colors import parse_colors
-    colors = parse_colors(spec)
+    from .colors import parse_colors
+    colors, transparent_background_colors = parse_colors(spec)
     profiles = window.screen.color_profile,
-    patch_color_profiles(colors, profiles, True)
+    patch_color_profiles(colors, transparent_background_colors, profiles, True)
 
 
 def parse_var(defn: Iterable[str]) -> Iterator[tuple[str, str]]:
@@ -512,8 +545,8 @@ force_window_launch = ForceWindowLaunch()
 non_window_launch_types = 'background', 'clipboard', 'primary'
 
 
-def parse_remote_control_passwords(allow_remote_control: bool, passwords: Sequence[str]) -> Optional[dict[str, Sequence[str]]]:
-    remote_control_restrictions: Optional[dict[str, Sequence[str]]] = None
+def parse_remote_control_passwords(allow_remote_control: bool, passwords: Sequence[str]) -> dict[str, Sequence[str]] | None:
+    remote_control_restrictions: dict[str, Sequence[str]] | None = None
     if allow_remote_control and passwords:
         from kitty.options.utils import remote_control_password
         remote_control_restrictions = {}
@@ -527,27 +560,38 @@ def _launch(
     boss: Boss,
     opts: LaunchCLIOptions,
     args: list[str],
-    target_tab: Optional[Tab] = None,
+    target_tab: Tab | None = None,
     force_target_tab: bool = False,
-    active: Optional[Window] = None,
     is_clone_launch: str = '',
-    rc_from_window: Optional[Window] = None,
-    base_env: Optional[dict[str, str]] = None,
-) -> Optional[Window]:
-    active = active or boss.active_window_for_cwd
-    if active:
-        active_child = active.child
-    else:
-        active_child = None
+    rc_from_window: Window | None = None,
+    base_env: dict[str, str] | None = None,
+) -> Window | None:
+    source_window = boss.active_window_for_cwd
+    if opts.source_window:
+        for qw in boss.match_windows(opts.source_window, rc_from_window):
+            source_window = qw
+            break
+    source_child = source_window.child if source_window else None
+    next_to = boss.active_window
+    if opts.next_to:
+        for qw in boss.match_windows(opts.next_to, rc_from_window):
+            next_to = qw
+            break
+    if target_tab and next_to and next_to not in target_tab:
+        next_to = None
     if opts.window_title == 'current':
-        opts.window_title = active.title if active else None
+        opts.window_title = source_window.title if source_window else None
     if opts.tab_title == 'current':
         atab = boss.active_tab
+        if source_window and (qt := source_window.tabref()):
+            atab = qt
         opts.tab_title = atab.effective_title if atab else None
     if opts.os_window_title == 'current':
         tm = boss.active_tab_manager
+        if source_window and (qt := source_window.tabref()) and (qr := qt.tab_manager_ref()):
+            tm = qr
         opts.os_window_title = get_os_window_title(tm.os_window_id) if tm else None
-    env = get_env(opts, active_child, base_env)
+    env = get_env(opts, source_child, base_env)
     kw: LaunchKwds = {
         'allow_remote_control': opts.allow_remote_control,
         'remote_control_passwords': parse_remote_control_passwords(opts.allow_remote_control, opts.remote_control_password),
@@ -571,25 +615,23 @@ def _launch(
         kw['bias'] = max(-100, min(opts.bias, 100))
     if opts.cwd:
         if opts.cwd == 'current':
-            if active:
-                kw['cwd_from'] = CwdRequest(active)
+            if source_window:
+                kw['cwd_from'] = CwdRequest(source_window)
         elif opts.cwd == 'last_reported':
-            if active:
-                kw['cwd_from'] = CwdRequest(active, CwdRequestType.last_reported)
+            if source_window:
+                kw['cwd_from'] = CwdRequest(source_window, CwdRequestType.last_reported)
         elif opts.cwd == 'oldest':
-            if active:
-                kw['cwd_from'] = CwdRequest(active, CwdRequestType.oldest)
+            if source_window:
+                kw['cwd_from'] = CwdRequest(source_window, CwdRequestType.oldest)
         elif opts.cwd == 'root':
-            if active:
-                kw['cwd_from'] = CwdRequest(active, CwdRequestType.root)
+            if source_window:
+                kw['cwd_from'] = CwdRequest(source_window, CwdRequestType.root)
         else:
             kw['cwd'] = opts.cwd
-        if kw['cwd_from'] is not None and rc_from_window is not None:
-            kw['cwd_from'].rc_from_window_id = rc_from_window.id
     if opts.location != 'default':
         kw['location'] = opts.location
-    if opts.copy_colors and active:
-        kw['copy_colors_from'] = active
+    if opts.copy_colors and source_window:
+        kw['copy_colors_from'] = source_window
     pipe_data: dict[str, Any] = {}
     if opts.stdin_source != 'none':
         q = str(opts.stdin_source)
@@ -599,25 +641,27 @@ def _launch(
                 q = f'@ansi_{q[1:]}'
         if opts.stdin_add_line_wrap_markers:
             q += '_wrap'
-        penv, stdin = boss.process_stdin_source(window=active, stdin=q, copy_pipe_data=pipe_data)
+        penv, stdin = boss.process_stdin_source(window=source_window, stdin=q, copy_pipe_data=pipe_data)
         if stdin:
             kw['stdin'] = stdin
             if penv:
                 env.update(penv)
 
     cmd = args or None
-    if opts.copy_cmdline and active_child:
-        cmd = active_child.foreground_cmdline
+    if opts.copy_cmdline and source_child:
+        cmd = source_child.foreground_cmdline
+    active = boss.active_window
     if cmd:
         final_cmd: list[str] = []
         for x in cmd:
-            if active and not opts.copy_cmdline:
+            if source_window and not opts.copy_cmdline:
                 if x == '@selection':
-                    s = boss.data_for_at(which=x, window=active)
+                    s = boss.data_for_at(which=x, window=source_window)
                     if s:
                         x = s
                 elif x == '@active-kitty-window-id':
-                    x = str(active.id)
+                    if active:
+                        x = str(active.id)
                 elif x == '@input-line-number':
                     if 'input_line_number' in pipe_data:
                         x = str(pipe_data['input_line_number'])
@@ -625,8 +669,8 @@ def _launch(
                     if 'lines' in pipe_data:
                         x = str(pipe_data['lines'])
                 elif x in ('@cursor-x', '@cursor-y', '@scrolled-by', '@first-line-on-screen', '@last-line-on-screen'):
-                    if active is not None:
-                        screen = active.screen
+                    if source_window is not None:
+                        screen = source_window.screen
                         if x == '@scrolled-by':
                             x = str(screen.scrolled_by)
                         elif x == '@cursor-x':
@@ -645,8 +689,10 @@ def _launch(
         kw['cmd'] = final_cmd
     if force_window_launch and opts.type not in non_window_launch_types:
         opts.type = 'window'
-    base_for_overlay = active
-    if target_tab:
+    if next_to and opts.type in non_window_launch_types:
+        next_to = None
+    base_for_overlay = next_to
+    if target_tab and (not base_for_overlay or base_for_overlay not in target_tab):
         base_for_overlay = target_tab.active_window
     if opts.type in ('overlay', 'overlay-main') and base_for_overlay:
         kw['overlay_for'] = base_for_overlay.id
@@ -663,37 +709,38 @@ def _launch(
         if stdin is not None:
             if opts.type == 'clipboard':
                 set_clipboard_string(stdin)
+                boss.handle_clipboard_loss('clipboard')
             else:
                 set_primary_selection(stdin)
+                boss.handle_clipboard_loss('primary')
     else:
         kw['hold'] = opts.hold
-        if force_target_tab:
+        if force_target_tab and target_tab is not None:
             tab = target_tab
         else:
-            tab = tab_for_window(boss, opts, target_tab)
-        if tab is not None:
-            watchers = load_watch_modules(opts.watcher)
-            with Window.set_ignore_focus_changes_for_new_windows(opts.keep_focus):
-                new_window: Window = tab.new_window(
-                    env=env or None, watchers=watchers or None, is_clone_launch=is_clone_launch, **kw)
-            if spacing:
-                patch_window_edges(new_window, spacing)
-                tab.relayout()
-            if opts.color:
-                apply_colors(new_window, opts.color)
-            if opts.keep_focus:
-                if active:
-                    boss.set_active_window(active, switch_os_window_if_needed=True, for_keep_focus=True)
-                if not Window.initial_ignore_focus_changes_context_manager_in_operation:
-                    new_window.ignore_focus_changes = False
-            if opts.logo:
-                new_window.set_logo(opts.logo, opts.logo_position or '', opts.logo_alpha)
-            if opts.type == 'overlay-main':
-                new_window.overlay_type = OverlayType.main
-            if opts.var:
-                for key, val in parse_var(opts.var):
-                    new_window.set_user_var(key, val)
-            return new_window
+            tab = tab_for_window(boss, opts, target_tab, next_to)
+        watchers = load_watch_modules(opts.watcher)
+        with Window.set_ignore_focus_changes_for_new_windows(opts.keep_focus):
+            new_window: Window = tab.new_window(
+                env=env or None, watchers=watchers or None, is_clone_launch=is_clone_launch, next_to=next_to, **kw)
+        if spacing:
+            patch_window_edges(new_window, spacing)
+            tab.relayout()
+        if opts.color:
+            apply_colors(new_window, opts.color)
+        if opts.keep_focus:
+            if active:
+                boss.set_active_window(active, switch_os_window_if_needed=True, for_keep_focus=True)
+            if not Window.initial_ignore_focus_changes_context_manager_in_operation:
+                new_window.ignore_focus_changes = False
+        if opts.logo:
+            new_window.set_logo(opts.logo, opts.logo_position or '', opts.logo_alpha)
+        if opts.type == 'overlay-main':
+            new_window.overlay_type = OverlayType.main
+        if opts.var:
+            for key, val in parse_var(opts.var):
+                new_window.set_user_var(key, val)
+        return new_window
     return None
 
 
@@ -701,18 +748,17 @@ def launch(
     boss: Boss,
     opts: LaunchCLIOptions,
     args: list[str],
-    target_tab: Optional[Tab] = None,
+    target_tab: Tab | None = None,
     force_target_tab: bool = False,
-    active: Optional[Window] = None,
     is_clone_launch: str = '',
-    rc_from_window: Optional[Window] = None,
-    base_env: Optional[dict[str, str]] = None,
-) -> Optional[Window]:
-    active = active or boss.active_window_for_cwd
+    rc_from_window: Window | None = None,
+    base_env: dict[str, str] | None = None,
+) -> Window | None:
+    active = boss.active_window
     if opts.keep_focus and active:
         orig, active.ignore_focus_changes = active.ignore_focus_changes, True
     try:
-        return _launch(boss, opts, args, target_tab, force_target_tab, active, is_clone_launch, rc_from_window, base_env)
+        return _launch(boss, opts, args, target_tab, force_target_tab, is_clone_launch, rc_from_window, base_env)
     finally:
         if opts.keep_focus and active:
             active.ignore_focus_changes = orig
@@ -722,7 +768,7 @@ def clone_safe_opts() -> frozenset[str]:
     return frozenset((
         'window_title', 'tab_title', 'type', 'keep_focus', 'cwd', 'env', 'var', 'hold',
         'location', 'os_window_class', 'os_window_name', 'os_window_title', 'os_window_state',
-        'logo', 'logo_position', 'logo_alpha', 'color', 'spacing',
+        'logo', 'logo_position', 'logo_alpha', 'color', 'spacing', 'next_to',
     ))
 
 
@@ -837,7 +883,7 @@ class EditCmd:
     def on_edit_window_close(self, window: Window) -> None:
         self.check_status()
 
-    def check_status(self, timer_id: Optional[int] = None) -> None:
+    def check_status(self, timer_id: int | None = None) -> None:
         if self.abort_signaled:
             return
         boss = get_boss()
@@ -873,7 +919,7 @@ class CloneCmd:
 
     def __init__(self, msg: str) -> None:
         self.args: list[str] = []
-        self.env: Optional[dict[str, str]] = None
+        self.env: dict[str, str] | None = None
         self.cwd = ''
         self.shell = ''
         self.envfmt = 'default'
@@ -908,6 +954,23 @@ class CloneCmd:
                     # skip SSH environment variables
                     'SSH_CLIENT', 'SSH_CONNECTION', 'SSH_ORIGINAL_COMMAND', 'SSH_TTY', 'SSH2_TTY',
                     'SSH_TUNNEL', 'SSH_USER_AUTH', 'SSH_AUTH_SOCK',
+                    # Dont clone KITTY_WINDOW_ID
+                    'KITTY_WINDOW_ID',
+                    # Bash variables from "bind -x" and "complete -C" (needed not to confuse bash-preexec)
+                    'READLINE_ARGUMENT', 'READLINE_LINE', 'READLINE_MARK', 'READLINE_POINT',
+                    'COMP_LINE', 'COMP_POINT', 'COMP_TYPE',
+                    # GPG gpg-agent
+                    'GPG_TTY',
+                    # Session variables of XDG
+                    'XDG_SESSION_CLASS', 'XDG_SESSION_ID', 'XDG_SESSION_TYPE',
+                    # Session variables of GNU Screen
+                    'STY', 'WINDOW',
+                    # Session variables of interactive shell plugins
+                    'ATUIN_SESSION', 'ATUIN_HISTORY_ID',
+                    'BLE_SESSION_ID', '_ble_util_fdlist_cloexec', '_ble_util_fdvars_export',
+                    '_GITSTATUS_CLIENT_PID', '_GITSTATUS_REQ_FD', '_GITSTATUS_RESP_FD', 'GITSTATUS_DAEMON_PID',
+                    'MCFLY_SESSION_ID',
+                    'STARSHIP_SESSION_KEY',
                 } and not k.startswith((
                     # conda state env vars for multi-level virtual environments
                     'CONDA_PREFIX_',
@@ -929,7 +992,8 @@ def remote_edit(msg: str, window: Window) -> None:
             q.abort_signaled = c.abort_signaled
         return
     cmdline = get_editor(path_to_edit=c.file_localpath, line_number=c.line_number)
-    w = launch(get_boss(), c.opts, cmdline, active=window)
+    c.opts.source_window = c.opts.next_to = f'id:{window.id}'
+    w = launch(get_boss(), c.opts, cmdline)
     if w is not None:
         c.source_window_id = window.id
         c.editor_window_id = w.id
@@ -990,4 +1054,6 @@ def clone_and_launch(msg: str, window: Window) -> None:
             cmdline[0] = window.child.final_exe
         if cmdline and cmdline == [window.child.final_exe] + window.child.argv[1:]:
             cmdline = window.child.unmodified_argv
-    launch(get_boss(), c.opts, cmdline, active=window, is_clone_launch=is_clone_launch)
+    c.opts.source_window = f'id:{window.id}'
+    c.opts.next_to = c.opts.next_to or c.opts.source_window
+    launch(get_boss(), c.opts, cmdline, is_clone_launch=is_clone_launch)
